searchData={"items":[{"type":"module","title":"SimpleStruct","doc":"`SimpleStruct` is a library to create structs in a concise way.\n\nThis is _not_ meant as a replacement of Elixir's `defstruct`. In fact, it uses\nit. It's meant to be used as a way to make it easier to define several structs\ninside a module.\n\nAnd if your struct needs to \"graduate\" at some point, you can easily extract\nit and expand it.","ref":"SimpleStruct.html"},{"type":"module","title":"Example - SimpleStruct","doc":"Take an example where you want to define a bunch of events in your\napplication.\n\nIdeally, you want each event to be a struct. That is our first-class data\nstructure in Elixir, and it allows things like safety when constructing it and\nbetter pattern matching.\n\nYou have 2 options:\n\n1. You have an explosion of files, each with a simple `defmodule` +\n`defstruct` in it, OR\n2. You have a really large module with tons of nested modules!\n\nBoth seem overkill for different reasons. So, maybe you take secret option (3)\nand you don't make them into structs! :(\n\nWith `SimpleStruct`, it's simple.\n\nJust `import SimpleStruct` and define all the structs you want in one place:\n\n```elixir\ndefmodule MyApp.Events do\n  import SimpleStruct\n\n  defdata UserLoggedIn, [:user_id, action: \"logged in\"]\n  defdata OrderPlaced, [:user_id, :order_id, required: true]\n  defdata OrderCancelled, [:user_id, :order_id, required: [:order_id]]\nend\n```","ref":"SimpleStruct.html#module-example"},{"type":"macro","title":"SimpleStruct.defdata/3","doc":"Creates a module and defines a struct inside it with then given fields.","ref":"SimpleStruct.html#defdata/3"},{"type":"macro","title":"Options - SimpleStruct.defdata/3","doc":"- `required`: defaults to empty. You can pass a list of keys that are required\nor `true` to require all keys.","ref":"SimpleStruct.html#defdata/3-options"},{"type":"macro","title":"Examples - SimpleStruct.defdata/3","doc":"```elixir\ndefmodule MyApp.Events do\n  import SimpleStruct\n\n  defdata UserLoggedIn, [:user_id, action: \"logged in\"]\n  defdata OrderPlaced, [:user_id, :order_id, required: true]\n  defdata OrderCancelled, [:user_id, :order_id, required: [:order_id]]\nend\n```\n\nThat will create three structs: `%Myapp.Events.UserLoggedIn{}`,\n`%Myapp.Events.OrderPlaced{}`, and `%Myapp.Events.OrderCancelled{}`.\n\n- `UserLoggedIn` will have a default `action` of `\"logged in\"`,\n- `OrderPlaced` will enforce that both `:user_id` and `:order_id` keys are\nprovided when creating the struct, and\n- `OrderCancelled` will only enforce `:order_id`\n\nIn addition, all modules will have a `new/1` helper function that can be used\nto create the structs by passing keyword lists.\n\n```elixir\nUserLoggedIn.new(user_id: 1)\n# => %Myapp.Events.UserLoggedIn{user_id: 1, action: \"logged in\"}\n```\n\nIn other words, it's the equivalent of doing this:\n\n```elixir\ndefmodule MyApp.Events do\n  defmodule UserLoggedIn do\n    defstruct [:user_id, action: \"logged in\"]\n\n    def new(attrs) do\n      struct!(__MODULE__, attrs)\n    end\n  end\n\n  defmodule OrderPlaced  do\n    @enforce_keys [:user_id, :order_id]\n    defstruct [:user_id, :order_id]\n\n    def new(attrs) do\n      struct!(__MODULE__, attrs)\n    end\n  end\n\n  defmodule OrderCancelled do\n    @enforce_keys [:order_id]\n    defstruct [:user_id, :order_id]\n\n    def new(attrs) do\n      struct!(__MODULE__, attrs)\n    end\n  end\nend\n```","ref":"SimpleStruct.html#defdata/3-examples"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}